@@ -73,7 +73,8 @@
 #include "gst/gst_private.h"
 
 #include "gsttypefindelement.h"
-#include "gst/gst-i18n-lib.h"
+#include "gstcoreelementselements.h"
+#include <glib/gi18n-lib.h>
 #include "gst/base/gsttypefindhelper.h"
 
 #include <gst/gsttypefind.h>
@@ -129,6 +130,8 @@ enum
 #define gst_type_find_element_parent_class parent_class
 G_DEFINE_TYPE_WITH_CODE (GstTypeFindElement, gst_type_find_element,
     GST_TYPE_ELEMENT, _do_init);
+GST_ELEMENT_REGISTER_DEFINE (typefind, "typefind", GST_RANK_NONE,
+    GST_TYPE_TYPE_FIND_ELEMENT);
 
 static void gst_type_find_element_dispose (GObject * object);
 static void gst_type_find_element_set_property (GObject * object,
@@ -527,6 +530,7 @@ gst_type_find_element_seek (GstTypeFindElement * typefind, GstEvent * event)
   }
 
   typefind->need_segment = TRUE;
+  typefind->seqnum = gst_event_get_seqnum (event);
 
   /* restart our task since it might have been stopped when we did the
    * flush. */
@@ -796,9 +800,10 @@ static gchar *
 gst_type_find_get_extension (GstTypeFindElement * typefind, GstPad * pad)
 {
   GstQuery *query;
-  gchar *uri, *result;
-  size_t len;
-  gint find;
+  gchar *uri, *result, *path, *base_path, *find;
+  GstUri *gst_uri;
+
+  base_path = NULL;
 
   query = gst_query_new_uri ();
 
@@ -810,24 +815,37 @@ gst_type_find_get_extension (GstTypeFindElement * typefind, GstPad * pad)
   if (uri == NULL)
     goto no_uri;
 
-  GST_DEBUG_OBJECT (typefind, "finding extension of %s", uri);
-
-  /* find the extension on the uri, this is everything after a '.' */
-  len = strlen (uri);
-  find = len - 1;
-
-  while (find >= 0) {
-    if (uri[find] == '.')
-      break;
-    find--;
-  }
-  if (find < 0)
+  /* data URIs paths are opaque and do not semantically represent a
+     filesystem-like resource path, so skip URI parsing for this case. */
+  if (g_str_has_prefix (uri, "data:"))
     goto no_extension;
 
-  result = g_strdup (&uri[find + 1]);
+  GST_DEBUG_OBJECT (typefind, "finding extension of %s", uri);
+
+  gst_uri = gst_uri_from_string (uri);
+  if (gst_uri == NULL)
+    goto invalid_uri;
+
+  path = gst_uri_get_path (gst_uri);
+  gst_uri_unref (gst_uri);
+
+  if (path == NULL)
+    goto invalid_uri;
+
+  base_path = g_path_get_basename (path);
+  g_free (path);
+
+  /* find the extension on the path, this is everything after a '.' */
+  find = strrchr (base_path, '.');
+
+  if (find == NULL)
+    goto no_extension;
+
+  result = g_strdup (find + 1);
 
   GST_DEBUG_OBJECT (typefind, "found extension %s", result);
   gst_query_unref (query);
+  g_free (base_path);
   g_free (uri);
 
   return result;
@@ -845,11 +863,19 @@ no_uri:
     gst_query_unref (query);
     return NULL;
   }
+invalid_uri:
+  {
+    GST_INFO_OBJECT (typefind, "failed to extract path from uri %s", uri);
+    g_free (uri);
+    gst_query_unref (query);
+    return NULL;
+  }
 no_extension:
   {
     GST_INFO_OBJECT (typefind, "could not find uri extension in %s", uri);
-    gst_query_unref (query);
+    g_free (base_path);
     g_free (uri);
+    gst_query_unref (query);
     return NULL;
   }
 }
@@ -1172,9 +1198,12 @@ gst_type_find_element_loop (GstPad * pad)
     GstBuffer *outbuf = NULL;
 
     if (typefind->need_segment) {
+      GstEvent *event;
       typefind->need_segment = FALSE;
-      gst_pad_push_event (typefind->src,
-          gst_event_new_segment (&typefind->segment));
+      event = gst_event_new_segment (&typefind->segment);
+      if (typefind->seqnum != 0)
+        gst_event_set_seqnum (event, typefind->seqnum);
+      gst_pad_push_event (typefind->src, event);
     }
 
     /* Pull 4k blocks and send downstream */
@@ -1253,6 +1282,7 @@ gst_type_find_element_activate_sink_mode (GstPad * pad, GstObject * parent,
         typefind->need_segment = TRUE;
         typefind->need_stream_start = TRUE;
         typefind->offset = 0;
+        typefind->seqnum = 0;
         res = TRUE;
       } else {
         res = gst_pad_stop_task (pad);
